@@ -8,8 +8,8 @@
 #' @description
 #'  \Sexpr[results=rd, stage=render]{lifecycle::badge("experimental")}
 #'
-#'  The values are paired/grouped such that the highest and lowest values
-#'  form the first group, the second highest and the second lowest values
+#'  The values are paired/grouped such that the lowest and highest values
+#'  form the first group, the second lowest and the second highest values
 #'  form the second group, and so on.
 #'  The values are then sorted by these groups/pairs.
 #'
@@ -71,15 +71,27 @@
 #' pair_extremes(df, col = "C")$C
 #'
 #' # Shuffle the members pair-wise
+#' # The rows within each pair are shuffled
+#' # while the `.pair` column maintains it order
 #' pair_extremes(df, col = "A", shuffle_members = TRUE)
 #'
 #' # Shuffle the order of the pairs
+#' # The rows within each pair maintain their order
+#' # and stay together but the `.pair` column is shuffled
 #' pair_extremes(df, col = "A", shuffle_pairs = TRUE)
 #'
 #' # Use recursive pairing
+#' # Mostly meaningful with much larger datasets
+#' # Order initial grouping by pair identifiers
 #' pair_extremes(df, col = "A", num_pairings = 2)
+#' # Order initial grouping by aggregate values
+#' pair_extremes(df, col = "A", num_pairings = 2, order_by_aggregates = TRUE)
 #'
 #' # Grouped by G
+#' # Each G group only has 3 elements
+#' # so it only creates 1 pair and a group
+#' # with the single excessive element
+#' # per G group
 #' df %>%
 #'   dplyr::select(G, A) %>% # For clarity
 #'   dplyr::group_by(G) %>%
@@ -88,23 +100,27 @@
 #' # Plot the extreme pairs
 #' plot(
 #'   x = 1:10,
-#'   y = pair_extremes(df, col = "B")$B
+#'   y = pair_extremes(df, col = "B")$B,
+#'   col = as.character(rep(1:5, each = 2))
 #' )
 #' # With shuffled pair members (run a few times)
 #' plot(
 #'   x = 1:10,
-#'   y = pair_extremes(df, col = "B", shuffle_members = TRUE)$B
+#'   y = pair_extremes(df, col = "B", shuffle_members = TRUE)$B,
+#'   col = as.character(rep(1:5, each = 2))
 #' )
 #' # With shuffled pairs (run a few times)
 #' plot(
 #'   x = rep(1:5, each = 2),
-#'   y = pair_extremes(df, col = "B", shuffle_pairs = TRUE)$B
+#'   y = pair_extremes(df, col = "B", shuffle_pairs = TRUE)$B,
+#'   col = as.character(rep(1:5, each = 2))
 #' )
 pair_extremes <- function(data,
                           col = NULL,
                           unequal_method = "middle",
                           num_pairings = 1,
                           balance = "mean",
+                          order_by_aggregates = FALSE,
                           shuffle_members = FALSE,
                           shuffle_pairs = FALSE,
                           factor_name = ifelse(num_pairings == 1, ".pair", ".pairing"),
@@ -115,6 +131,7 @@ pair_extremes <- function(data,
     unequal_method = unequal_method,
     num_pairings = num_pairings,
     balance = balance,
+    order_by_aggregates = order_by_aggregates,
     shuffle_members = shuffle_members,
     shuffle_pairs = shuffle_pairs,
     factor_name = factor_name,
@@ -128,6 +145,7 @@ pair_extremes_vec <- function(data,
                               unequal_method = "middle",
                               num_pairings = 1,
                               balance = "mean",
+                              order_by_aggregates = FALSE,
                               shuffle_members = FALSE,
                               shuffle_pairs = FALSE){
   checkmate::assert(checkmate::check_vector(data, strict = TRUE),
@@ -137,6 +155,7 @@ pair_extremes_vec <- function(data,
     unequal_method = unequal_method,
     num_pairings = num_pairings,
     balance = balance,
+    order_by_aggregates = order_by_aggregates,
     shuffle_members = shuffle_members,
     shuffle_pairs = shuffle_pairs,
     factor_name = NULL,
